import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:print_bluetooth_thermal/print_bluetooth_thermal.dart';
import 'package:shared_preferences/shared_preferences.dart';

class PrinterManager {
  List<BluetoothInfo>? devicesInfo;
  BluetoothInfo? selectedBluetoothInfo;
  bool? connected;
  BuildContext context;
  String? printerWindowRoute;

  PrinterManager({
    required this.context,
  });

  Future<void> connect(BluetoothInfo bluetoothInfo) async {
    connected = await PrintBluetoothThermal.connectionStatus;
    if (connected == true) {
      disconnect();
    }
    connected = await PrintBluetoothThermal.connect(
        macPrinterAddress: bluetoothInfo.macAdress);
    if (connected == true) {
      selectedBluetoothInfo = bluetoothInfo;
      saveToLocalStorage(bluetoothInfo);
    }
  }

  Future<void> disconnect() async {
    bool success = await PrintBluetoothThermal.disconnect;
    if (success == true) {
      connected = false;
      selectedBluetoothInfo = null;
      removeFromLocalStorage();
    }
  }

  Future<void> saveToLocalStorage(BluetoothInfo bluetoothInfo) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await prefs.setString(
      'printer-settings',
      jsonEncode(
        {'macAddress': bluetoothInfo.macAdress, 'name': bluetoothInfo.name},
      ),
    );
  }

  Future<void> loadFromLocalStorage() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String? printerSettingsJsonStr = prefs.getString('printer-settings');
    if (![null, ''].contains(printerSettingsJsonStr)) {
      Map<String, dynamic> tmpMap =
          jsonDecode(printerSettingsJsonStr.toString());

      await connect(
        BluetoothInfo(
          name: tmpMap['name'],
          macAdress: tmpMap['macAddress'],
        ),
      );
    }
  }

  Future<void> removeFromLocalStorage() async {
    final preferences = await SharedPreferences.getInstance();
    await preferences.remove('printer-settings');
  }

  Future<List<BluetoothInfo>?> getDevicesInfo(BuildContext context) async {
    final List<BluetoothInfo> listResult =
        await PrintBluetoothThermal.pairedBluetooths;

    if (listResult.isEmpty) {
      showDialog(
        context: context,
        builder: (BuildContext context) {
          return const AlertDialog(
            title: Text("Nenhum dispositivo vinculado"),
            content: Text(
                "Acesse as configurações, ative o bluetooth e conecte a impressora."),
          );
        },
      );
    }

    return listResult;
  }

  Future<void> print(List<int> textToPrint, BuildContext context,
      void Function()? onPrintEnd) async {
    connected = await PrintBluetoothThermal.connectionStatus;
    if (connected != true) {
      await loadFromLocalStorage();
    }

    if (connected == true) {
      bool connectionStatus = await PrintBluetoothThermal.connectionStatus;
      if (connectionStatus) {
        await PrintBluetoothThermal.writeBytes(textToPrint);
        if (onPrintEnd != null) onPrintEnd();
      }
    } else {
      if (![null, ''].contains(printerWindowRoute)) {
        Navigator.of(context).pushNamed(printerWindowRoute!, arguments: {
          'tabIdx': 1,
          'textToPrint': textToPrint,
          'onPrintEnd': onPrintEnd
        });
      }
    }
  }
}
