import 'package:flutter/material.dart';
import 'package:print_bluetooth_thermal/print_bluetooth_thermal.dart';
import 'package:provider/provider.dart';
import 'package:sl7_fwork_flutter/printer/printer_manger.dart';
import 'package:sl7_fwork_flutter/providers/snack.dart';
import 'package:sl7_fwork_flutter/widgets/inputFormContainer/input_form_container_item.dart';

class PrinterSettingsWidget extends StatefulWidget {
  final PrinterManager printerManager;
  final List<int>? textToPrint;
  final void Function()? onPrintEnd;
  final ButtonStyle? buttonStyle;
  final double? maxWidth;

  const PrinterSettingsWidget({
    Key? key,
    required this.printerManager,
    this.textToPrint,
    this.onPrintEnd,
    this.buttonStyle,
    this.maxWidth,
  }) : super(key: key);

  @override
  State<PrinterSettingsWidget> createState() => _PrinterSettingsWidgetState();
}

class _PrinterSettingsWidgetState extends State<PrinterSettingsWidget> {
  List<BluetoothInfo> devices = [];
  BluetoothInfo? selectedBlueToothInfo;

  @override
  void initState() {
    widget.printerManager.getDevicesInfo(context).then((devices) {
      setState(() {
        this.devices = devices ?? [];
      });
    });
    widget.printerManager.loadFromLocalStorage().then((value) {
      if (widget.printerManager.connected == true) {
        setState(() {
          selectedBlueToothInfo = widget.printerManager.selectedBluetoothInfo;
        });
      }
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Center(
      child: SingleChildScrollView(
        child: Container(
          padding: const EdgeInsets.only(
            right: 10,
            bottom: 10,
          ),
          constraints: BoxConstraints(
            maxWidth: widget.maxWidth ?? 350,
          ),
          child: Column(children: [
            InputFormItemWidget(
              child: ElevatedButton(
                child: const Text('ATUALIZAR DISPOSITIVOS'),
                onPressed: () {
                  widget.printerManager.getDevicesInfo(context).then((devices) {
                    setState(() {
                      this.devices = devices ?? [];
                    });
                  });
                },
                style: widget.buttonStyle,
              ),
            ),
            if (devices.isNotEmpty)
              InputFormItemWidget(
                child: DropdownButtonFormField<String>(
                  value: selectedBlueToothInfo?.macAdress,
                  items: devices.map((blueToothInfo) {
                    return DropdownMenuItem<String>(
                      child:
                          Text(blueToothInfo.name, textAlign: TextAlign.center),
                      value: blueToothInfo.macAdress,
                    );
                  }).toList(),
                  onChanged: (blueToothInfoMac) {
                    setState(() {
                      selectedBlueToothInfo = devices.firstWhere(
                          (element) => element.macAdress == blueToothInfoMac);
                    });
                    widget.printerManager.connect(selectedBlueToothInfo!);
                  },
                ),
              ),
            InputFormItemWidget(
              child: ElevatedButton(
                child: const Text('DESCONECTAR'),
                onPressed: () async {
                  await widget.printerManager.disconnect();
                  if (widget.printerManager.connected != true) {
                    setState(() {
                      selectedBlueToothInfo = null;
                    });
                  }
                },
                style: widget.buttonStyle,
              ),
            ),
            InputFormItemWidget(
              child: ElevatedButton(
                child: const Text('IMPRIMIR'),
                onPressed: () {
                  if (widget.printerManager.connected == true) {
                    widget.printerManager.print(
                        widget.textToPrint ?? [], context, widget.onPrintEnd);
                  } else {
                    Provider.of<SnackProvider>(context, listen: false).show(
                        context: context,
                        text: 'Nenhum dispositivo conectado',
                        snackType: SnackType.warning);
                  }
                },
                style: widget.buttonStyle,
              ),
            ),
          ]),
        ),
      ),
    );
  }
}
