import 'package:flutter/material.dart';

class FloatingWidget extends StatefulWidget {
  final Widget button;
  final Widget Function(BuildContext context, OverlayEntry? entry) builder;
  final void Function()? onTap;

  const FloatingWidget({
    Key? key,
    required this.button,
    required this.builder,
    this.onTap,
  }) : super(key: key);

  @override
  State<FloatingWidget> createState() => _FloatingWidgetState();
}

class _FloatingWidgetState extends State<FloatingWidget> {
  OverlayEntry? entry;
  Offset offset = const Offset(20, 40);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        InkWell(
          child: widget.button,
          onTap: () {
            if (entry?.mounted == true) return;
            entry = OverlayEntry(
              builder: (context) {
                return Positioned(
                  left: offset.dx,
                  top: offset.dy,
                  child: GestureDetector(
                    onPanUpdate: ((details) {
                      offset += details.delta;
                      entry!.markNeedsBuild();
                    }),
                    child: Card(
                        color: Colors.transparent,
                        elevation: 0.0,
                        child: widget.builder(context, entry)),
                  ),
                );
              },
            );
            final overlay = Overlay.of(context)!;
            overlay.insert(entry!);
            if (widget.onTap != null) {
              widget.onTap!();
            }
          },
        ),
      ],
    );
  }
}
