import 'package:flutter/material.dart';

enum HorizontalScrollButtonType {
  left,
  right,
}

class HorizontalScrollButtonWidget extends StatelessWidget {
  final void Function()? onTap;
  final HorizontalScrollButtonType horizontalScrollButtonType;
  final ScrollController scrollController;
  final double scrollSize;
  final double? iconSize;

  const HorizontalScrollButtonWidget({
    Key? key,
    this.onTap,
    required this.horizontalScrollButtonType,
    required this.scrollController,
    required this.scrollSize,
    this.iconSize,
  }) : super(key: key);

  _moveLeft() {
    scrollController.animateTo(scrollController.offset - scrollSize,
        curve: Curves.easeIn, duration: const Duration(milliseconds: 400));
  }

  _moveRight() {
    scrollController.animateTo(scrollController.offset + scrollSize,
        curve: Curves.easeIn, duration: const Duration(milliseconds: 400));
  }

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        Opacity(
          opacity: .5,
          child: Container(
            // height: height ?? 50,
            // width: width ?? 40,
            decoration: BoxDecoration(
              color: Colors.grey,
              borderRadius:
                  horizontalScrollButtonType == HorizontalScrollButtonType.left
                      ? const BorderRadius.only(
                          topRight: Radius.circular(50),
                          bottomRight: Radius.circular(50),
                        )
                      : const BorderRadius.only(
                          topLeft: Radius.circular(50),
                          bottomLeft: Radius.circular(50),
                        ),
            ),
            child: HorizontalScrollButtonIconWidget(
              horizontalScrollButtonType: horizontalScrollButtonType,
              size: iconSize,
            ),
          ),
        ),
        Material(
          color: Colors.transparent,
          child: InkWell(
            splashColor: Colors.grey[100]!.withOpacity(0.2),
            borderRadius:
                horizontalScrollButtonType == HorizontalScrollButtonType.left
                    ? const BorderRadius.only(
                        topRight: Radius.circular(50),
                        bottomRight: Radius.circular(50),
                      )
                    : const BorderRadius.only(
                        topLeft: Radius.circular(50),
                        bottomLeft: Radius.circular(50),
                      ),
            onTap: () {
              if (horizontalScrollButtonType ==
                  HorizontalScrollButtonType.left) {
                _moveLeft();
              } else if (horizontalScrollButtonType ==
                  HorizontalScrollButtonType.right) {
                _moveRight();
              }

              if (onTap != null) {
                onTap!();
              }
            },
            child: HorizontalScrollButtonIconWidget(
              horizontalScrollButtonType: horizontalScrollButtonType,
              size: iconSize,
            ),
          ),
        )
      ],
    );
  }
}

class HorizontalScrollButtonIconWidget extends StatelessWidget {
  final double? size;
  const HorizontalScrollButtonIconWidget({
    Key? key,
    required this.horizontalScrollButtonType,
    this.size,
  }) : super(key: key);

  final HorizontalScrollButtonType horizontalScrollButtonType;

  @override
  Widget build(BuildContext context) {
    return Icon(
      horizontalScrollButtonType == HorizontalScrollButtonType.left
          ? Icons.arrow_left
          : Icons.arrow_right,
      color: Colors.white,
      size: size ?? 35,
    );
  }
}
