import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:sl7_fwork_flutter/widgets/inputFormContainer/input_form_control_label.dart';

enum InputWidgetType {
  text,
  num,
  pwd,
}

class InputWidget extends StatelessWidget {
  final String? label;
  final Color? labelColor;
  final InputWidgetType? type;
  final double? width;
  final TextEditingController? controller;
  final void Function(String)? onChanged;
  final String? hintText;
  final Widget? prefixIcon;
  final Widget? suffixIcon;
  final List<TextInputFormatter>? inputFormatters;
  final Color? fillColor;
  final bool obscureText;
  final String? initialValue;
  final bool? noRadius;
  final void Function(bool)? onFocusChange;
  final void Function(String)? onFieldSubmitted;
  final bool readOnly;
  final FocusNode? nextFocus;

  const InputWidget({
    Key? key,
    this.label,
    this.labelColor,
    this.type = InputWidgetType.text,
    this.width,
    this.controller,
    this.onChanged,
    this.hintText,
    this.prefixIcon,
    this.suffixIcon,
    this.inputFormatters,
    this.fillColor,
    this.obscureText = false,
    this.initialValue,
    this.noRadius,
    this.onFocusChange,
    this.onFieldSubmitted,
    this.readOnly = false,
    this.nextFocus,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        if (![null, ''].contains(label))
          InputFormControlLabelWidget(
            label: label.toString(),
            color: labelColor,
          ),
        SizedBox(
          width: width,
          height: 40,
          child: Focus(
            onFocusChange: onFocusChange,
            child: TextFormField(
              readOnly: readOnly,
              textInputAction: TextInputAction.next,
              onFieldSubmitted: (value) {
                if (onFieldSubmitted != null)
                {
                  onFieldSubmitted!(value);
                }
                if (nextFocus != null) {
                  FocusScope.of(context).requestFocus(nextFocus);
                }
              },
              initialValue: initialValue,
              inputFormatters: inputFormatters,
              keyboardType: type == InputWidgetType.num
                  ? TextInputType.number
                  : TextInputType.text,
              obscureText: obscureText,
              controller: controller,
              onChanged: onChanged,
              cursorColor: Colors.grey,
              decoration: InputDecoration(
                prefixIcon: prefixIcon,
                suffixIcon: suffixIcon,
                hintText: hintText,
                hintStyle: const TextStyle(color: Colors.grey),
                fillColor: fillColor ?? Colors.white,
                filled: true,
                contentPadding:
                    const EdgeInsets.only(top: 20, left: 10, right: 10),
                enabledBorder: OutlineInputBorder(
                  borderRadius:
                      BorderRadius.circular(noRadius == true ? 0 : 5.0),
                  borderSide: const BorderSide(color: Colors.transparent),
                ),
                focusedBorder: OutlineInputBorder(
                  borderRadius:
                      BorderRadius.circular(noRadius == true ? 0 : 5.0),
                  borderSide: const BorderSide(color: Colors.transparent),
                ),
              ),
            ),
          ),
        ),
      ],
    );
  }
}
