import 'package:flutter/material.dart';
import 'package:sl7_fwork_flutter/widgets/wait/spinnerContent/spinner_content.dart';

class InputSelectWidget<T> extends StatelessWidget {
  final double width;
  final T? value;
  final List<T> items;
  final void Function(T?)? onChanged;
  final String? hintText;
  final String? label;
  final Color? labelColor;
  final Widget Function(T item)? itemBuilder;
  final bool? showProgrees;
  final Color? progressColor;

  const InputSelectWidget({
    Key? key,
    required this.width,
    required this.items,
    this.value,
    this.onChanged,
    this.hintText,
    this.label,
    this.labelColor,
    this.itemBuilder,
    this.showProgrees,
    this.progressColor,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        Column(
          children: [
            if (![null, ''].contains(label))
              Align(
                alignment: Alignment.centerLeft,
                child: Text(
                  label.toString(),
                  style: TextStyle(
                    color: labelColor,
                  ),
                ),
              ),
            if (![null, ''].contains(label))
              const SizedBox(
                height: 10,
              ),
            SizedBox(
              width: width,
              height: 40,
              child: DropdownButtonFormField<T>(
                decoration: InputDecoration(
                  hintText: hintText,
                  hintStyle: const TextStyle(
                    color: Colors.grey,
                  ),
                  fillColor: Colors.white,
                  filled: true,
                  contentPadding: items.isNotEmpty
                      ? const EdgeInsets.only(left: 10, right: 10)
                      : null,
                  enabledBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(5.0),
                    borderSide: const BorderSide(color: Colors.transparent),
                  ),
                  focusedBorder: OutlineInputBorder(
                    borderRadius: BorderRadius.circular(5.0),
                    borderSide: const BorderSide(color: Colors.transparent),
                  ),
                  labelStyle: const TextStyle(
                    color: Colors.black,
                  ),
                  border: const OutlineInputBorder(),
                ),
                value: value,
                items: items
                    .map(
                      (e) => DropdownMenuItem<T>(
                        value: e,
                        child: itemBuilder != null
                            ? itemBuilder!(e)
                            : (T is String)
                                ? Text(e.toString())
                                : const SizedBox.shrink(),
                      ),
                    )
                    .toList(),
                onChanged: onChanged,
              ),
            ),
          ],
        ),
        if (showProgrees == true)
          Positioned(
            top: 36,
            right: 40,
            child: SpinnerContentWidget(
              spinnerHeight: 20,
              spinnerWidth: 20,
              color: progressColor,
            ),
          ),
      ],
    );
  }
}
