import 'package:flutter/material.dart';

class RadioListItem<T> {
  String? label;
  Widget? title;
  T value;

  RadioListItem({
    this.label,
    this.title,
    required this.value,
  });
}

class RadioListWidget extends StatefulWidget {
  final List<RadioListItem> items;
  final dynamic initValue;
  final void Function(RadioListItem item, dynamic value)? onChange;

  const RadioListWidget({
    Key? key,
    required this.items,
    this.initValue,
    this.onChange,
  }) : super(key: key);

  @override
  State<RadioListWidget> createState() => _RadioListWidgetState();
}

class _RadioListWidgetState<T> extends State<RadioListWidget> {
  T? groupValue;

  @override
  void initState() {
    groupValue = widget.initValue;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      children: widget.items.map((item) {
        return ListTile(
          title: item.title ?? Text(item.label.toString()),
          leading: Radio<dynamic>(
            value: item.value,
            groupValue: groupValue,
            onChanged: (value) {
              setState(() {
                groupValue = value;
              });
              if (widget.onChange != null) {
                widget.onChange!(item, value);
              }
            },
          ),
        );
      }).toList(),
    );
  }
}
